package com.loit.common.script;

import com.loit.common.script.dto.BackendFrontEnum;
import com.loit.common.script.dto.DeployInfoDataDTO;
import com.loit.common.script.dto.FrontGroupEnum;
import com.loit.common.script.dto.FrontInstanceDto;
import com.loit.common.utils.ListUtil;
import com.loit.common.utils.StringUtils;
import com.loit.common.utils.excel.ImportExcel;
import com.loit.common.utils.file.FileUtils;
import com.loit.common.utils.freemarker.FreeMarkerUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.entity.ContentType;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

@Slf4j
public class GeneratorPushScript {


    protected static String root_path = "F:\\9Git140\\loit-build-common\\loit-build-component\\loit-build-deploy-env\\src\\main\\resources\\bin";

    protected static String root_path_jenkins_full = root_path + "\\jenkinsPush";

    private static Map<String, FrontInstanceDto> frontInstanceDtoMap = new LinkedHashMap<>();


    public static void main(String[] args) {

        try {

            String filePathStr = "F:\\9Git140\\loit-build-common\\loit-build-component\\loit-build-deploy-env\\src\\main\\resources\\deployInfo.xlsx";

            File pdfFile = new File(filePathStr);
            FileInputStream fileInputStream = new FileInputStream(pdfFile);
            MultipartFile multipartFile = new MockMultipartFile(pdfFile.getName(), pdfFile.getName(),

                    ContentType.APPLICATION_OCTET_STREAM.toString(), fileInputStream);
            ImportExcel ei = new ImportExcel(multipartFile, 1, 0);
            List<DeployInfoDataDTO> grayVersionDataDTOList = ei.getDataList(DeployInfoDataDTO.class);

            if (ListUtil.isEmpty(grayVersionDataDTOList)) {
                return;
            }


            //对前端实例进行分组
            for (DeployInfoDataDTO deployDataDTO : grayVersionDataDTOList) {


                String needCreateInstanceSql = deployDataDTO.getNeedCreateInstanceSql();
                if ("no".equals(needCreateInstanceSql)) {
                    continue;
                }

                BackendFrontEnum backendFrontType = BackendFrontEnum.getEnumByCode(deployDataDTO.getBackendFrontType());


                String serviceName = deployDataDTO.getServiceName();
                FrontInstanceDto frontInstanceDto = frontInstanceDtoMap.get(serviceName);
                if (frontInstanceDto == null) {
                    frontInstanceDto = new FrontInstanceDto();
                    String serviceNameSub = "loit-" + serviceName.replace("-web", "").replace("-h5", "");
                    String serviceNameCookie = "loit" + serviceName.replaceAll("-", "");
                    frontInstanceDto.setServiceName(serviceName);
                    frontInstanceDto.setServiceNameSub(serviceNameSub);
                    frontInstanceDto.setServiceNameCookie(serviceNameCookie);
                    frontInstanceDto.setServiceNameAbb(deployDataDTO.getServiceNameAbb());
                    frontInstanceDto.setGitPath(deployDataDTO.getGitPath());
                    frontInstanceDto.setGitBuildSub(deployDataDTO.getGitBuildSub());
                    frontInstanceDto.setBackendFrontType(backendFrontType);
                    frontInstanceDto.setDeployJar(deployDataDTO.getDeployJar());
                    frontInstanceDtoMap.put(serviceName, frontInstanceDto);
                }

                FrontGroupEnum frontGroup = FrontGroupEnum.getEnumByCode(deployDataDTO.getFrontGroup());
                String ipAndPort = deployDataDTO.getFixedIp() + ":" + deployDataDTO.getPort();
                if (FrontGroupEnum.gray.equals(frontGroup)) {
                    frontInstanceDto.getGrayList().add(ipAndPort);
                }

                if (FrontGroupEnum.normal.equals(frontGroup)) {
                    frontInstanceDto.getNormalList().add(ipAndPort);
                }
            }


            //生成前端nginx 配置文件
            frontInstanceDtoMap.forEach((key, frontInstanceDto) -> {
                if (StringUtils.isBlank(key)) {
                    return;
                }

                buildInitEnv(frontInstanceDto);

            });

        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
    }

    /**
     * 生成InitEnv
     *
     * @throws IOException
     */
    private static void buildInitEnv(FrontInstanceDto frontInstanceDto) {

        String fileDir = root_path_jenkins_full;

        File folder = new File(fileDir);
        if (!folder.exists()) {
            folder.mkdirs();
        }

        String serviceName = frontInstanceDto.getServiceName();

        if (StringUtils.isBlank(serviceName)) {
            return;
        }

        String createJobFile = root_path_jenkins_full + "\\createJob.txt";
        String jenkinsJobName = createJenkinsJobName(serviceName);

        Map model = new HashMap();
        model.put("deployJobName", jenkinsJobName);


        String result = FreeMarkerUtils.process("push\\jenkinsJobCreateZww.ftl", model);
        FileUtils.appendNewLine(createJobFile, result);

        String jobsDir = root_path_jenkins_full + "\\" + jenkinsJobName;
        File jobsFolder = new File(jobsDir);
        if (!jobsFolder.exists()) {
            jobsFolder.mkdirs();
        }

        String configFilePath = jobsDir + "\\config.xml";
        String jobConfig = FreeMarkerUtils.process("push\\zww-push-config.ftl", model);
        FileUtils.appendNewLine(configFilePath, jobConfig);

    }

    private static String createJenkinsJobName(String serviceName) {

        if ("getway".equals(serviceName)) {
            serviceName = "gateway";
        }

        String jenkinsJobName = "loit-push-jar-" + serviceName;
        return jenkinsJobName;
    }

}
