package com.loit.component.keygen.leaf;

import com.loit.component.keygen.leaf.core.reg.RegistryCenter;
import org.apache.commons.lang3.StringUtils;

import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Properties;

/**
 * leafKey 获取不同的keyGenerator
 */
public class LeafSegmentKeyGeneratorFactory {

    private Map<String, LeafSegmentKeyGenerator> keyGeneratorMap = new LinkedHashMap<>();

    private Properties globleProperties = new Properties();

    private RegistryCenter registryCenter;

    /**
     * 根据 leafKey 标识不同的keyGenerator
     */
    public synchronized LeafSegmentKeyGenerator getKeyGenerator(String leafKey) {

        if (StringUtils.isEmpty(leafKey)) {
            return null;
        }

        LeafSegmentKeyGenerator leafSegmentKeyGenerator = keyGeneratorMap.get(leafKey);
        if (leafSegmentKeyGenerator == null) {
            leafSegmentKeyGenerator = new LeafSegmentKeyGenerator();
            Properties properties = new Properties();
            properties.setProperty("leaf.namespace", globleProperties.getProperty("leaf.namespace", LeafSegmentKeyGenerator.DEFAULT_NAMESPACE));
            properties.setProperty("server.list", globleProperties.getProperty("server.list"));
            properties.setProperty("leaf.segment.id.initial.value", globleProperties.getProperty("leaf.segment.id.initial.value", LeafSegmentKeyGenerator.DEFAULT_INITIAL_VALUE));
            properties.setProperty("leaf.segment.step", globleProperties.getProperty("leaf.segment.step", LeafSegmentKeyGenerator.DEFAULT_STEP));
            properties.setProperty("leaf.key", leafKey);
            properties.setProperty("registry.center.type", globleProperties.getProperty("registry.center.type", LeafSegmentKeyGenerator.DEFAULT_REGISTRY_CENTER));
            leafSegmentKeyGenerator.setProperties(properties);

            if (registryCenter == null) {
                registryCenter = leafSegmentKeyGenerator.buidLeafRegistryCenter();
            } else {
                leafSegmentKeyGenerator.setLeafRegistryCenter(registryCenter);
            }

            keyGeneratorMap.put(leafKey, leafSegmentKeyGenerator);
        }

        return leafSegmentKeyGenerator;
    }


    public Properties getGlobleProperties() {
        return globleProperties;
    }

    public void setGlobleProperties(Properties globleProperties) {
        this.globleProperties = globleProperties;
    }
}
