package io.opensharding.keygen.leaf;

import com.sankuai.inf.leaf.IDGen;
import com.sankuai.inf.leaf.snowflake.SnowflakeIDGenImpl;
import lombok.Getter;
import lombok.Setter;
import org.apache.shardingsphere.spi.keygen.ShardingKeyGenerator;

import java.util.Properties;

/**
 * Key generator implemented by leaf snowflake algorithms.
 */
public final class LeafSnowflakeKeyGenerator implements ShardingKeyGenerator {

    public final static String TYPE = "LEAF_SNOWFLAKE";

    @Getter
    @Setter
    private Properties properties;

    private IDGen idGen;

    @Override
    public Comparable<?> generateKey() {
        if (this.idGen == null) {
            synchronized (this) {
                if (this.idGen == null) {
                    IDGen idGen = new SnowflakeIDGenImpl(properties.getProperty(LeafPropertiesConstant.LEAF_ZK_LIST), 8089);
                    this.idGen = idGen;
                }
            }
        }
        return this.idGen.get(properties.getProperty(LeafPropertiesConstant.LEAF_KEY)).getId();
    }

    @Override
    public String getType() {
        return TYPE;
    }

    @Override
    public Properties getProperties() {
        return this.properties;
    }

    @Override
    public void setProperties(Properties properties) {
        this.properties = properties;
    }
}
